@@ -20,43 +20,54 @@ rfPermute.default <- function(x, y, ..., nrep = 100, num.cores = 1) {
   rf.call$importance <- TRUE
   rf.call[-1] <- lapply(as.list(rf.call[-1]), eval, envir = parent.frame())
   rf <- eval(rf.call)
-  imp.names <- colnames(rf$importance)
+  rf$call <- orig.call
   
   # permutes 'y' in rf.call 'nrep' times and runs randomForest  
   if(nrep > 0) {
     # define permutation function
+    #   returns a 3-dimensional array of unscaled and scaled importance scores
     permFunc <- function(y, perm.rf.call) {
       rf.call$y <- y
-      eval(rf.call)$importance
+      perm.rf <- eval(rf.call)
+      imp <- perm.rf$importance
+      impSD <- perm.rf$importanceSD
+      rm(perm.rf)
+      imp.arr <- .makeImpArray(imp, 2, c("unscaled", "scaled"))
+      imp.arr[, , 2] <- .scaleImp(imp, impSD)
+      return(imp.arr)
     }
     
     # create list of permuted y values
     ran.y <- lapply(1:nrep, function(i) sample(rf.call$y))
     
     # get importance scores for permutations
-    importance.perm <- NULL
+    #  a list of 3-dimensional arrays of importance scores
+    null.dist<- NULL
     if(num.cores > 1) {
       cl <- makeForkCluster(num.cores)
       tryCatch({
-        importance.perm <- parLapply(cl, ran.y, permFunc, perm.rf.call = rf.call)
-      }, finally = stopCluster(cl))
+        null.dist <- parLapply(cl, ran.y, permFunc, perm.rf.call = rf.call)
+      }, finally = {
+        stopCluster(cl)
+        closeAllConnections()
+      })
     } else {
-      importance.perm <- lapply(ran.y, permFunc)
+      null.dist <- lapply(ran.y, permFunc)
     }
     
-    # create null distribution for each variable  
-    rf$null.dist <- sapply(imp.names, function(imp.type) {
-      null.dist <- sapply(1:length(importance.perm), function(i) importance.perm[[i]][, imp.type, drop = FALSE])
-      null.dist <- t(rbind(null.dist))
-      colnames(null.dist) <- rownames(rf$importance)
-      null.dist
-    }, simplify = FALSE)
+    # create and load null distribution arrays for each scaled and unscaled importances
+    rf$null.dist <- list(unscaled = .makeImpArray(rf$importance, nrep, NULL))
+    rf$null.dist$scaled <- rf$null.dist$unscaled
+    for(i in 1:nrep) {
+      rf$null.dist$unscaled[, , i] <- null.dist[[i]][, , "unscaled"]
+      rf$null.dist$scaled[, , i] <- null.dist[[i]][, , "scaled"]
+    }
     
     # calculate p-value of observed importance metrics
-    rf$null.dist$pval <- calc.imp.pval(rf, imp.names) 
-  } 
+    rf$pval <- .calcImpPval(rf) 
+    
+    class(rf) <- c("rfPermute", "randomForest")
+  }
   
-  rf$call <- orig.call
-  class(rf) <- c("rfPermute", "randomForest")
   return(rf)  
 }
\ No newline at end of file
